"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _ChunkRegistry_registry;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChunkRegistry = void 0;
/**
 * InMemoryPool is a subset of ChunkPool abstraction
 * This pool saves rows in chunks that are stored in process memory
 *
 * @warning storing data in process memory can lead you to LOSS OF CONSISTANCE
 * but instead of this disadvantage you gain the cheapest and the most simple way to work
 * with clickhouse
 *
 * Data loss can only occur in two cases:
 * 1. When OS sends to a process SIGKILL code which is killing your process without grace
 * 2. When some piece of data contains anomalies such as `undefined` etc
 */
class ChunkRegistry {
    /**
     * Create InMemoryPool instance
     */
    constructor() {
        _ChunkRegistry_registry.set(this, void 0);
        __classPrivateFieldSet(this, _ChunkRegistry_registry, {
            tableChunk: {},
            chunkTable: {},
            chunks: {}
        }, "f");
    }
    increaseSize(id, delta) {
        __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks[id].size += delta;
        __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks[id].chunkRef.size += delta;
    }
    decreaseSize(id, delta) {
        __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks[id].size -= delta;
        __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks[id].chunkRef.size -= delta;
    }
    register(chunk) {
        __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunkTable[chunk.id] = chunk.table;
        __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").tableChunk[chunk.table] = chunk.id;
        __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks[chunk.id] = {
            chunkRef: chunk,
            expiresAt: chunk.expiresAt,
            size: chunk.size
        };
    }
    unregister(id) {
        if (__classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks[id]) {
            const table = __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunkTable[id];
            delete __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").tableChunk[table];
            delete __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunkTable[id];
            delete __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks[id];
        }
    }
    getOne(id) {
        return __classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks[id];
    }
    getAll() {
        return Object.values(__classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks);
    }
    isEmpty() {
        return Object.keys(__classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks).length === 0;
    }
    getTables() {
        return Object.keys(__classPrivateFieldGet(this, _ChunkRegistry_registry, "f").tableChunk);
    }
    getChunks() {
        return Object.keys(__classPrivateFieldGet(this, _ChunkRegistry_registry, "f").chunks);
    }
}
exports.ChunkRegistry = ChunkRegistry;
_ChunkRegistry_registry = new WeakMap();
