"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _ChunkResolver_instances, _ChunkResolver_dataWatcher, _ChunkResolver_registry, _ChunkResolver_options, _ChunkResolver_handlers, _ChunkResolver_toResolveQueue, _ChunkResolver_watchingToResolveQueue, _ChunkResolver_watchingRegistry, _ChunkResolver_commandStop, _ChunkResolver_startWatchingRegistry, _ChunkResolver_watchRegistry, _ChunkResolver_startWatchingToResolveQueue, _ChunkResolver_watchToResolveQueue, _ChunkResolver_setupShutdownHandler, _ChunkResolver_startWatching;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChunkResolver = void 0;
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable no-async-promise-executor */
const cuid_1 = __importDefault(require("cuid"));
const chunk_registry_1 = require("./chunk-registry");
const queue_1 = require("./queue/queue");
const scratch_chunk_1 = require("./chunk/scratch-chunk");
const disk_watcher_1 = require("./watchers/disk.watcher");
const process_watcher_1 = require("./watchers/process.watcher");
const chunk_facade_1 = require("./chunk/chunk-facade");
const errors_1 = require("./errors");
const utils_1 = require("./utils");
const config_error_1 = require("./errors/config.error");
/**
 * ChunkResolver is the central element of the application,
 * which implements all the logic of working with the caching process
 * and provides an API for interacting "outside"
 */
class ChunkResolver {
    /**
     * Create ChunkResolver instance
     *
     * @param {ResolverOptions} options
     */
    constructor(options) {
        _ChunkResolver_instances.add(this);
        _ChunkResolver_dataWatcher.set(this, void 0);
        _ChunkResolver_registry.set(this, void 0);
        _ChunkResolver_options.set(this, void 0);
        _ChunkResolver_handlers.set(this, void 0);
        _ChunkResolver_toResolveQueue.set(this, void 0);
        _ChunkResolver_watchingToResolveQueue.set(this, void 0);
        _ChunkResolver_watchingRegistry.set(this, void 0);
        _ChunkResolver_commandStop.set(this, void 0);
        const registry = new chunk_registry_1.ChunkRegistry();
        let dataWatcher;
        switch (options.dataWatcher) {
            case 'disk':
                if (!options.disk) {
                    throw new config_error_1.ConfigError("E_CONFIG_PARAM_REQUIRED" /* E_CODES.E_CONFIG_PARAM_REQUIRED */);
                }
                dataWatcher = new disk_watcher_1.DiskWatcher(registry, {
                    disk: options.disk,
                    chunkSize: options.chunkSize
                });
                break;
            case 'process':
                dataWatcher = new process_watcher_1.ProcessWatcher(registry, {
                    chunkSize: options.chunkSize
                });
                break;
        }
        /**
         * Registry is a brain of the package
         * It stores metadata and relations and shares it to every part of system
         */
        __classPrivateFieldSet(this, _ChunkResolver_registry, registry, "f");
        /**
         * Data watcher is "the hands" off the package
         * It allows to this class to interact with stored data (memory, disk or cloud)
         */
        __classPrivateFieldSet(this, _ChunkResolver_dataWatcher, dataWatcher, "f");
        __classPrivateFieldSet(this, _ChunkResolver_options, options, "f");
        __classPrivateFieldSet(this, _ChunkResolver_watchingToResolveQueue, false, "f");
        __classPrivateFieldSet(this, _ChunkResolver_watchingRegistry, false, "f");
        __classPrivateFieldSet(this, _ChunkResolver_commandStop, false, "f");
        __classPrivateFieldSet(this, _ChunkResolver_handlers, new Map(), "f");
        /**
         * There is defined a queue to proceed chunks consistently
         */
        __classPrivateFieldSet(this, _ChunkResolver_toResolveQueue, new queue_1.Queue(), "f");
        /**
         * On init we have to restore already existing data from watcher
         * and start pending
         */
        dataWatcher.restore().then(async () => {
            await __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_startWatching).call(this);
        });
        /**
         * Enable graceful shutdown
         */
        __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_setupShutdownHandler).call(this);
    }
    /**
     * Pass your data in this method to store it
     *
     * It registers data in the system registry and saves it though the chosen data watcher
     *
     * @warning This method does not guarantee the validation of the transmitted data,
     * which may subsequently affect the outcome of inserting a chunk into the DBMS.
     * Be confident to validate it inself
     *
     * @param {Table} table table name
     * @param {InsertRow[]} rows list of rows
     * @returns
     */
    async cache(table, rows) {
        while (rows.length > 0) {
            let chunk = __classPrivateFieldGet(this, _ChunkResolver_registry, "f").getAll().find(state => state.chunkRef.isUnblocked())?.chunkRef;
            if (!chunk) {
                chunk = new scratch_chunk_1.ScratchChunk(__classPrivateFieldGet(this, _ChunkResolver_dataWatcher, "f"), {
                    table,
                    liveAtLeastMs: __classPrivateFieldGet(this, _ChunkResolver_options, "f").chunkLifeMs
                });
                __classPrivateFieldGet(this, _ChunkResolver_registry, "f").register(chunk);
            }
            const rowCountToOverfill = Math.abs(__classPrivateFieldGet(this, _ChunkResolver_options, "f").chunkSize - chunk.size);
            const chunkRows = rows.splice(0, rowCountToOverfill);
            await __classPrivateFieldGet(this, _ChunkResolver_dataWatcher, "f").save({
                chunkRef: chunk,
                insertRows: chunkRows
            });
        }
        __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_startWatching).call(this);
    }
    /**
     * It registers a hadnler for resolved chunks
     * Pass @sync or @async callback and use it to save data to your DBMS
     */
    onResolved(onResolved) {
        __classPrivateFieldGet(this, _ChunkResolver_handlers, "f").set((0, cuid_1.default)(), onResolved);
    }
}
exports.ChunkResolver = ChunkResolver;
_ChunkResolver_dataWatcher = new WeakMap(), _ChunkResolver_registry = new WeakMap(), _ChunkResolver_options = new WeakMap(), _ChunkResolver_handlers = new WeakMap(), _ChunkResolver_toResolveQueue = new WeakMap(), _ChunkResolver_watchingToResolveQueue = new WeakMap(), _ChunkResolver_watchingRegistry = new WeakMap(), _ChunkResolver_commandStop = new WeakMap(), _ChunkResolver_instances = new WeakSet(), _ChunkResolver_startWatchingRegistry = 
/**
 * A method to start a loop to check there are chunks in our registry
 */
async function _ChunkResolver_startWatchingRegistry() {
    if (!__classPrivateFieldGet(this, _ChunkResolver_watchingRegistry, "f")) {
        __classPrivateFieldSet(this, _ChunkResolver_watchingRegistry, true, "f");
        await __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_watchRegistry).call(this);
    }
}, _ChunkResolver_watchRegistry = 
/**
 * It searches in a loop for come chunks which are ready to be resolved
 * The chunk is considered to be resolved if:
 * - it is expired by the time
 * - it has reached the size limit
 * - it is consistent (i.e it has no running async operations currently)
 *
 * When the chunk matches conditions,
 * the system removes it from registry and puts to a resolve queue
 */
async function _ChunkResolver_watchRegistry() {
    while (!__classPrivateFieldGet(this, _ChunkResolver_registry, "f").isEmpty() || __classPrivateFieldGet(this, _ChunkResolver_commandStop, "f")) {
        const snapshot = __classPrivateFieldGet(this, _ChunkResolver_registry, "f").getAll();
        for (const state of snapshot) {
            const isReady = state.chunkRef.isReady();
            const isConsistent = state.chunkRef.isConsistent();
            const canBlock = isReady;
            const canResolve = isConsistent && isReady;
            if (canBlock) {
                state.chunkRef.block();
            }
            if (canResolve) {
                __classPrivateFieldGet(this, _ChunkResolver_registry, "f").unregister(state.chunkRef.id);
                __classPrivateFieldGet(this, _ChunkResolver_toResolveQueue, "f").enqueue(state.chunkRef);
                __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_startWatchingToResolveQueue).call(this);
            }
            await (0, utils_1.sleep)(__classPrivateFieldGet(this, _ChunkResolver_options, "f").checkIntervalMs);
        }
    }
    __classPrivateFieldSet(this, _ChunkResolver_watchingRegistry, false, "f");
}, _ChunkResolver_startWatchingToResolveQueue = 
/**
 * A method to start a loop to pass chunks outside and resolve
 */
async function _ChunkResolver_startWatchingToResolveQueue() {
    if (!__classPrivateFieldGet(this, _ChunkResolver_watchingToResolveQueue, "f")) {
        __classPrivateFieldSet(this, _ChunkResolver_watchingToResolveQueue, true, "f");
        await __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_watchToResolveQueue).call(this);
    }
}, _ChunkResolver_watchToResolveQueue = 
/**
 * It handles chunks in a loop, one by one from queue
 * To pass chunk outside and then resolves it
 *
 * Resolve is a required process to clenup chunk data from storage (process memory, disk space or cloud)
 */
async function _ChunkResolver_watchToResolveQueue() {
    while (!__classPrivateFieldGet(this, _ChunkResolver_toResolveQueue, "f").isEmpty() || __classPrivateFieldGet(this, _ChunkResolver_commandStop, "f")) {
        const chunk = __classPrivateFieldGet(this, _ChunkResolver_toResolveQueue, "f").dequeue();
        if (!chunk) {
            continue;
        }
        const handlers = [...__classPrivateFieldGet(this, _ChunkResolver_handlers, "f").entries()];
        if (!handlers.length) {
            /**
             * there is no any handler to pass rows
             * stop next steps to avoid data lose
             */
            throw new errors_1.CacheError("E_NO_HANDLER" /* E_CODES.E_NO_HANDLER */);
        }
        // return to queue if chunk has inconsistent state
        if (!chunk.isConsistent()) {
            __classPrivateFieldGet(this, _ChunkResolver_toResolveQueue, "f").enqueue(chunk);
            continue;
        }
        while (!chunk.isConsistent()) {
            continue;
        }
        /**
         * Invoke chunk to all registered handlers
         */
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const handlerPromises = handlers.map(([_handlerId, handlerFunction]) => {
            /**
             * Convert sync callback to a Promise
             */
            if (handlerFunction.constructor.name === 'AsyncFunction') {
                return handlerFunction(new chunk_facade_1.ChunkFacade(chunk));
            }
            else {
                return new Promise((resolve) => {
                    resolve(handlerFunction(new chunk_facade_1.ChunkFacade(chunk)));
                });
            }
        });
        await Promise.allSettled(handlerPromises)
            .finally(async () => {
            await chunk.resolve();
        });
    }
    __classPrivateFieldSet(this, _ChunkResolver_watchingToResolveQueue, false, "f");
}, _ChunkResolver_setupShutdownHandler = async function _ChunkResolver_setupShutdownHandler() {
    const handleSignal = (signal) => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const signalHandler = (code) => {
            console.log(`Initialize chunks backup due to received ${signal} signal`);
            // set pending off
            __classPrivateFieldSet(this, _ChunkResolver_commandStop, true, "f");
            // Syncronously backup runtime data
            __classPrivateFieldGet(this, _ChunkResolver_dataWatcher, "f").backup();
            process.exit(0);
        };
        process.on(signal, signalHandler);
    };
    handleSignal('SIGTERM');
    handleSignal('SIGINT');
}, _ChunkResolver_startWatching = 
/**
 * Service method to start pending
 */
async function _ChunkResolver_startWatching() {
    await __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_startWatchingRegistry).call(this);
    await __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_startWatchingToResolveQueue).call(this);
};
