"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Chunk_dataWatcher;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Chunk = void 0;
/**
 * Chunk of a nasic item of this package
 *
 * Every chunk:
 * - has its unique id
 * - belongs to a table
 * - contain rows
 * - has size and time restrictions
 * - can be blocked/unblocked
 */
class Chunk {
    /**
     * Create Chunk instance
     *
     * @param {DataWatcher} dataWatcher
     * @param {ChunkData} data
     */
    constructor(dataWatcher, data) {
        _Chunk_dataWatcher.set(this, void 0);
        __classPrivateFieldSet(this, _Chunk_dataWatcher, dataWatcher, "f");
        this.id = data.id;
        this.size = data.size;
        this.blocked = false;
        this.consistent = true;
        this.forceReady = false;
        this.createdAt = Date.now();
        this.expiresAt = data.expiresAt;
        this.table = data.table;
    }
    isExpired() {
        return Date.now() >= this.expiresAt;
    }
    isOverfilled() {
        return __classPrivateFieldGet(this, _Chunk_dataWatcher, "f").countRows(this.id) >= __classPrivateFieldGet(this, _Chunk_dataWatcher, "f").$options.chunkSize;
    }
    isConsistent() {
        return this.consistent;
    }
    isUnblocked() {
        return !this.blocked;
    }
    isReady() {
        return this.forceReady || this.isExpired() || this.isOverfilled();
    }
    block() {
        this.blocked = true;
    }
    unblock() {
        this.blocked = false;
    }
    setConsistency(state) {
        console.log(this.id, { setConsistency: state });
        this.consistent = state;
    }
    setReady() {
        this.forceReady = true;
    }
    async loadRows() {
        return __classPrivateFieldGet(this, _Chunk_dataWatcher, "f")
            .load(this.id)
            .then(load => load.loadedRows);
    }
    async resolve() {
        await __classPrivateFieldGet(this, _Chunk_dataWatcher, "f").cleanup(this.id);
    }
}
exports.Chunk = Chunk;
_Chunk_dataWatcher = new WeakMap();
