/**
 * A generic class representing a Queue data structure.
 */
export declare class Queue<T> {
    private elements;
    /**
     * Creates an empty Queue.
     */
    constructor();
    /**
     * Adds an element to the end of the Queue.
     * @param element The element to enqueue.
     */
    enqueue(element: T): void;
    /**
     * Removes and returns the element at the front of the Queue.
     * @returns The element removed from the Queue, or undefined if the Queue is empty.
     */
    dequeue(): T | undefined;
    /**
     * Returns the element at the front of the Queue without removing it.
     * @returns The element at the front of the Queue, or undefined if the Queue is empty.
     */
    peek(): T | undefined;
    /**
     * Checks if the Queue is empty.
     * @returns true if the Queue is empty, false otherwise.
     */
    isEmpty(): boolean;
    /**
     * Returns the number of elements in the Queue.
     * @returns The number of elements in the Queue.
     */
    size(): number;
    /**
     * Removes all elements from the Queue, making it empty.
     */
    clear(): void;
}
